package com.lqkj.link.module.authority.service;

import com.lqkj.link.module.authority.domain.UserInfo;
import com.lqkj.link.module.authority.repository.RoleInfoRepository;
import com.lqkj.link.module.authority.repository.UserInfoRepository;
import jakarta.annotation.PostConstruct;
import org.springframework.transaction.annotation.Transactional;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Service
public class UserInfoService {
    private final UserInfoRepository userInfoRepository;
    private final PasswordEncoder passwordEncoder;
    private final RoleInfoRepository roleInfoRepository;

    public UserInfoService(UserInfoRepository userInfoRepository, PasswordEncoder passwordEncoder, RoleInfoRepository roleInfoRepository) {
        this.userInfoRepository = userInfoRepository;
        this.passwordEncoder = passwordEncoder;
        this.roleInfoRepository = roleInfoRepository;
    }

    @PostConstruct
    public void initSuperUser() {
        if (!userInfoRepository.existsById(1)) {
            userInfoRepository.initSuperAdmin();
            userInfoRepository.initSuperAdminRole();
        }
    }

    public UserInfo detailByUserCode(String userCode) {
        UserInfo userInfo = userInfoRepository.findByUserCode(userCode);
        userInfo.setHasAuth(userInfoRepository.hasAuth(userInfo.getUserId()));
        return userInfo;
    }

    public List<Map<String, Object>> findAuthListWithUserCode(String userCode) {
        List<Map<String, Object>> resultList = new ArrayList<>();
        UserInfo userInfo = userInfoRepository.findByUserCode(userCode);
        if (userInfo.getHasManage()) {
            List<Map<String, Object>> authList = userInfoRepository.queryAuthWithUserCode(userCode);

            Map<Integer, List<Map<String, Object>>> childrenMap = new HashMap<>();
            for (Map<String, Object> auth : authList) {
                Map<String, Object> result = new HashMap<>(auth);
                Map<String, Object> metaObject = new HashMap<>();
                metaObject.put("icon", auth.get("icon"));
                metaObject.put("title", auth.get("title"));
                result.put("meta", metaObject);

                if (auth.get("parent_id") == null) {
                    result.put("children", childrenMap.get(auth.get("authority_id")));
                    resultList.add(result);
                } else {
                    List<Map<String, Object>> childrenList = childrenMap.getOrDefault(auth.get("parent_id"), new ArrayList<>());
                    childrenList.add(result);
                    childrenMap.put((Integer) auth.get("parent_id"), childrenList);
                }
            }
        }

        return resultList;
    }

    public Page<UserInfo> pageQuery(String userCode, String displayName, Integer page, Integer pageSize) {
        Pageable pageable = PageRequest.of(page, pageSize);
        Page<UserInfo> userInfos = userInfoRepository.pageQueryV2(userCode, displayName, pageable);
        for (UserInfo userInfo : userInfos.getContent()) {
            userInfo.setRoleNames(roleInfoRepository.findRoleNames(userInfo.getUserId()));
        }
        return userInfos;
    }

    public UserInfo detail(Integer userId) {
        UserInfo userInfo = userInfoRepository.findById(userId).get();
        userInfo.setRoleInfoList(roleInfoRepository.listWithUserId(userId));
        return userInfo;
    }

    @Transactional
    public String save(UserInfo userInfo) {

        // 账号判断，8位数字
        String userCodeRegex = "\\d{8}";
        Pattern pattern = Pattern.compile(userCodeRegex);
        Matcher matcher = pattern.matcher(userInfo.getUserCode());
        if (!matcher.find()) {
            return "账号必须是8位数字！";
        }
        // 显示名称判断，2-8位中文字符
        String displayNameRegex = "[\u4E00-\u9FA5]{2,8}";
        pattern = Pattern.compile(displayNameRegex);
        matcher = pattern.matcher(userInfo.getDisplayName());
        if (!matcher.find()) {
            return "名称必须是2-8位中文字符！";
        }

        // 授权码判断，8位数字、大写字母、小写字母
        String auCodeRegex = "^(?=.*[a-z])(?=.*[A-Z])(?=.*[0-9]).{8}$";
        pattern = Pattern.compile(auCodeRegex);
        matcher = pattern.matcher(userInfo.getAuthorizationCode());
        if (!matcher.find()) {
            return "授权码必须是8位数字、大写字母、小写字母！";
        }

        if (userInfo.getUserId() == null) {
            // 重复判断
            if (userInfoRepository.hasSameUserCode(userInfo.getUserCode())) {
                return "账号与已有账号重复，请设置其他账号！";
            }
            // 密码判断，8-16位数字、大写字母、小写字母
            String passwordRegex = "^(?=.*\\d)(?=.*[a-z])(?=.*[A-Z])[a-zA-Z0-9]{8,16}$";
            pattern = Pattern.compile(passwordRegex);
            matcher = pattern.matcher(userInfo.getPassword());
            if (!matcher.find()) {
                return "密码必须是8-16位数字、大写字母、小写字母！";
            }

            userInfo.setLocking(false);
            userInfo.setPassword(passwordEncoder.encode(userInfo.getPassword()));
            userInfo.setUpdateTime(new Date());
            userInfo.setRefreshResource(false);
            userInfo = userInfoRepository.save(userInfo);
        } else {
            if (StringUtils.isNotBlank(userInfo.getPassword())) {
                // 密码判断，8-16位数字、大写字母、小写字母
                String passwordRegex = "^(?=.*\\d)(?=.*[a-z])(?=.*[A-Z])[a-zA-Z0-9]{8,16}$";
                pattern = Pattern.compile(passwordRegex);
                matcher = pattern.matcher(userInfo.getPassword());
                if (!matcher.find()) {
                    return "密码必须是8-16位数字、大写字母、小写字母！";
                }
            }
            UserInfo oldUser = userInfoRepository.findById(userInfo.getUserId()).get();
            if (StringUtils.isNotBlank(userInfo.getPassword())) {
                oldUser.setPassword(passwordEncoder.encode(userInfo.getPassword()));
            }
            oldUser.setDisplayName(userInfo.getDisplayName());
            oldUser.setHasManage(userInfo.getHasManage());
            oldUser.setAuthorizationCode(userInfo.getAuthorizationCode());
            oldUser.setUpdateTime(new Date());
            userInfoRepository.save(oldUser);
        }
        roleInfoRepository.updateUserRole(userInfo.getUserId(), userInfo.getRoleIds());
        return null;
    }

    public void delete(List<Integer> userIds) {
        userInfoRepository.deleteAllByIdInBatch(userIds);
    }

    public void updateInfo(String userCode, String displayName, String headImgPath) {
        UserInfo userInfo = userInfoRepository.findByUserCode(userCode);
        if (StringUtils.isNotBlank(displayName)) {
            userInfo.setDisplayName(displayName);
        }
        if ((StringUtils.isNotBlank(headImgPath))) {
            userInfo.setHeadImg(headImgPath);
        }
        userInfoRepository.save(userInfo);
    }

    public Boolean checkRefreshResource(String userCode) {
        return userInfoRepository.findByUserCode(userCode).getRefreshResource();
    }

    public void resourceRefreshed(String userCode) {
        userInfoRepository.resourceRefreshed();
    }
}
